package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.dragoninfo.dcuc.common.annotation.CacheLock;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.api.IApproveApiBusiness;
import com.dragonsoft.dcuc.approve.business.external.IBuSyncBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.business.external.INoticeBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.bu.TaskStatusEnum;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.bu.BuApproveSyncBizApproveReqVO;
import com.dragonsoft.dcuc.approve.model.req.bu.BuApproveSyncReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.report.ReportApproveSyncBizApproveReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.report.ReportApproveSyncBizGridReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.report.ReportApproveSyncReqVO;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.persistent.transaction.TransactionSupport;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/7
 */
@Slf4j
@Component
public class BuSyncBusinessImpl implements IBuSyncBusiness {

    private IApproveRecordService approveRecordService;

    private IApproveCirculationService approveCirculationService;

    private INoticeBusiness noticeBusiness;

    private IApproveApiBusiness approveApiBusiness;
    
    private IDcucBusiness dcucBusiness;

    @Autowired
    public void setApproveApiBusiness(IApproveApiBusiness approveApiBusiness) {
        this.approveApiBusiness = approveApiBusiness;
    }

    @Autowired
    public void setNoticeBusiness(INoticeBusiness noticeBusiness) {
        this.noticeBusiness = noticeBusiness;
    }

    @Autowired
    public void setApproveCirculationService(IApproveCirculationService approveCirculationService) {
        this.approveCirculationService = approveCirculationService;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }
    
    @Autowired
	public void setDcucBusiness(IDcucBusiness dcucBusiness) {
		this.dcucBusiness = dcucBusiness;
	}

    @Override
    @CacheLock
    public ResponseStatus syncTuBu() {
        TransactionSupport.execute(() -> {
            List<ApproveRecordInfo> unSyncList = approveRecordService.getUnSyncList();
            for (ApproveRecordInfo approveRecordInfo : unSyncList) {
                String id = approveRecordInfo.getId();
                String approveNo = approveRecordInfo.getApproveNo();
                Date startTime = approveRecordInfo.getStartTime();
                Date endTime = approveRecordInfo.getEndTime();

                //  检查任务编号是否正确,且开始、结束时间不为空
                boolean isNeedSync = approveApiBusiness.isStanderTaskNo(approveNo)
                        && ObjectUtil.isNotNull(startTime) && ObjectUtil.isNotNull(endTime);

                if (isNeedSync) {
                    ResponseStatus responseStatus = syncToBu(approveRecordInfo);
                    BooleanEnum booleanEnum = BooleanEnum.TRUE;
                    if (ResponseUtil.isFail(responseStatus)) {
                        log.error("同步到BU出错:{}", responseStatus.getMessage());
                        booleanEnum = BooleanEnum.FALSE;
                    }
                    approveRecordService.updateSyncFlag(id, booleanEnum);

                } else {
                    // 非标准的任务ID默认设置为成功
                    approveRecordService.updateSyncFlag(id, BooleanEnum.TRUE);
                }
            }
        });
        return ResponseStatus.success();
    }

    /**
     * 同步到BU
     *
     * @param approveRecordInfo 请求同步请求
     * @return 状态
     */
    @Transactional(rollbackFor = Throwable.class)
    @Override
    public ResponseStatus syncToBu(ApproveRecordInfo approveRecordInfo) {
        Assert.notNull(approveRecordInfo);
        ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();

        String id = approveRecordInfo.getId();

        BuApproveSyncReqVO buApproveSyncReqVO = new BuApproveSyncReqVO();

        ApproveCirculationInfo approveCirculationInfo = approveCirculationService.getListByApproveRecordLastAgree(id);
        Assert.notNull(approveCirculationInfo);
        // 设置最终审批人信息
        buApproveSyncReqVO.setLastReviewerId(approveCirculationInfo.getOperateUserId());
        buApproveSyncReqVO.setLastReviewerIdCard(approveCirculationInfo.getOperateUserId());
        buApproveSyncReqVO.setLastReviewerName(approveCirculationInfo.getOperateUserName());
        buApproveSyncReqVO.setLastReviewerUnitCode(approveCirculationInfo.getOperateUserOrgId());
        buApproveSyncReqVO.setLastReviewerUnitName(approveCirculationInfo.getOperateUserOrg());

        return syncToBu(id, buApproveSyncReqVO, approvalInfoDTO);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public ResponseStatus syncToBu(ApproveMessageInfoDTO approveMessageInfoDTO) {
        ApprovalInfoDTO approvalInfoDTO = approveMessageInfoDTO.getApprovalInfoDTO();
        String id = approvalInfoDTO.getId();

        BuApproveSyncReqVO buApproveSyncReqVO = new BuApproveSyncReqVO();

        SecurityUser securityUser = approveMessageInfoDTO.getSecurityUser();

        Assert.notNull(securityUser);
        // 设置最终审批人信息
        buApproveSyncReqVO.setLastReviewerIdCard(securityUser.getIdcard());
        buApproveSyncReqVO.setLastReviewerName(securityUser.getName());
        buApproveSyncReqVO.setLastReviewerUnitCode(securityUser.getSecurityOrg());
        buApproveSyncReqVO.setLastReviewerUnitName(securityUser.getSecurityOrgName());
        buApproveSyncReqVO.setTaskId(approvalInfoDTO.getApproveNo());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());

        return syncToBu(id, buApproveSyncReqVO, approvalInfoDTO);
    }

    /**
     * 同步到 BU
     *
     * @param id                 审批单ID
     * @param buApproveSyncReqVO 请求
     * @param approvalInfoDTO    审批单信息
     * @return 状态
     */
    protected ResponseStatus syncToBu(String id, BuApproveSyncReqVO buApproveSyncReqVO, ApprovalInfoDTO approvalInfoDTO) {
        buApproveSyncReqVO.setAppId(approvalInfoDTO.getAppCode());
        buApproveSyncReqVO.setAppName(approvalInfoDTO.getAppName());
        buApproveSyncReqVO.setCreateTime(approvalInfoDTO.getCreateTime());
        buApproveSyncReqVO.setFinishTime(approvalInfoDTO.getUpdateTime());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());
        buApproveSyncReqVO.setTaskId(approvalInfoDTO.getApproveNo());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());

        BuApproveSyncBizApproveReqVO buApproveSyncBizApproveReqVO = parseApprovalInfoDTO(approvalInfoDTO);

        Map<String, Object> bizData = new LinkedHashMap<>(2);
        bizData.put(ApproveConstants.TASK_INFO, buApproveSyncBizApproveReqVO);
        buApproveSyncReqVO.setBizData(bizData);
        
        
     // 报送到 BU 废掉了
//        ResponseStatus responseStatus = noticeBusiness.retryRequestSyncToBu(buApproveSyncReqVO);
        
     // 报送到 BU V2版本
        com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO basrVO = createTestV2(buApproveSyncReqVO);
        ResponseStatus responseStatus = noticeBusiness.retryRequestSyncToBuV2(basrVO);
        
        if (ResponseUtil.isSuccess(responseStatus)) {
            approveRecordService.updateSyncFlag(id, BooleanEnum.TRUE);
            return ResponseStatus.success();
        }
        approveRecordService.updateSyncFlag(id, BooleanEnum.FALSE);
        return ResponseStatus.fail();
    }
    // 20241106 add 本地审批调整成BU V2 方案 start 
    private com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO createTestV2(BuApproveSyncReqVO reportApproveSyncReqVO) {
		
    	SimpleDateFormat formatter = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
    	SimpleDateFormat formatterDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    	com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO basrVO = new com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO();
        
    	String taskId = reportApproveSyncReqVO.getTaskId();
    	Map<String, Object> bizData = reportApproveSyncReqVO.getBizData();
    	ReportApproveSyncBizApproveReqVO bizApproveReqVO = parseToReportApproveSyncBizApproveReqVO(bizData);
    	List<ReportApproveSyncBizGridReqVO> bizGridReqList = parseToReportApproveSyncBizGridReqVO(bizData);
        
        Map<String,Object> bizDataV2 = new HashMap<>();
        Map<String,Object> taskInfo = new HashMap<>();
        taskInfo.put("title", bizApproveReqVO.getTitle());//任务名称
        taskInfo.put("taskClass", bizApproveReqVO.getTaskClass());//任务类型编码
        try {
			taskInfo.put("taskTermStart", formatterDate.format(formatter.parse(bizApproveReqVO.getTaskTermStart())));//任务有效期开始时间
			taskInfo.put("taskTermEnd", formatterDate.format(formatter.parse(bizApproveReqVO.getTaskTermEnd())));//任务有效期结束时间
		} catch (ParseException e) {
			e.printStackTrace();
		}
        taskInfo.put("taskStatus", bizApproveReqVO.getTaskStatus());//任务状态
        List<Object> approveGrid = new ArrayList<>();
        
        for (ReportApproveSyncBizGridReqVO reportApproveSyncBizGridReqVO : bizGridReqList) {
        	Map<String,Object> g1 = new HashMap<>();
            g1.put("id", reportApproveSyncBizGridReqVO.getId());//要素或承办人数据唯一标识
            g1.put("dataType", reportApproveSyncBizGridReqVO.getDataType());//业务数据类型
            g1.put("elementContent", reportApproveSyncBizGridReqVO.getElementContent());//要素内容
            g1.put("elementType", reportApproveSyncBizGridReqVO.getElementType());//要输类型编码
            g1.put("elementTypeName", reportApproveSyncBizGridReqVO.getElementType());//要输类型名称
            g1.put("elementTermStart", reportApproveSyncBizGridReqVO.getElementTermStart());//要素有效期开始时间
            g1.put("elementTermEnd", reportApproveSyncBizGridReqVO.getElementTermEnd());//要素有效期结束时间
            g1.put("handleUnitCode", reportApproveSyncBizGridReqVO.getHandleUnitCode());//承办人所在组织机构编码
            g1.put("handleUnitName", reportApproveSyncBizGridReqVO.getHandleUnitName());//承办人所在组织机构名称
            g1.put("handleUserIdCard", reportApproveSyncBizGridReqVO.getHandleUserId());//承办人身份证号
            g1.put("handleUserName", reportApproveSyncBizGridReqVO.getHandleUserName());//承办人姓名
            approveGrid.add(g1);
		}
        
        bizDataV2.put("taskInfo", taskInfo);
        bizDataV2.put("approveGrid",approveGrid);
        
        String createrId = reportApproveSyncReqVO.getCreaterId();
        OauthUserVO oauthUserVO = dcucBusiness.getOauthUserInfoByIdcard(createrId);
        
        basrVO.setAppId(reportApproveSyncReqVO.getAppId());
        basrVO.setSourceOrgCode(oauthUserVO.getOrgCode());
        basrVO.setTaskId(taskId);
        String customFormat = "yyyy-MM-dd HH:mm:ss";
        basrVO.setCreateTime(DateUtil.format(reportApproveSyncReqVO.getCreateTime(), customFormat));
        basrVO.setCreatorIdCard(oauthUserVO.getIdcard());
        basrVO.setCreatorUserName(oauthUserVO.getName());
        basrVO.setCreatorUnitCode(oauthUserVO.getOrgCode());
        basrVO.setCreatorUnitName(oauthUserVO.getOrgName());
        basrVO.setLastReviewerIdCard(reportApproveSyncReqVO.getLastReviewerIdCard());
        basrVO.setLastReviewerName(reportApproveSyncReqVO.getLastReviewerName());
        basrVO.setLastReviewerUnitCode(reportApproveSyncReqVO.getLastReviewerUnitCode());
        basrVO.setLastReviewerUnitName(reportApproveSyncReqVO.getLastReviewerUnitName());
        basrVO.setFinishTime(DateUtil.format(reportApproveSyncReqVO.getFinishTime(), customFormat));
        
        
        basrVO.setBizData(bizDataV2);
        
        return basrVO;
	}
    
    /**
     * 转换任务信息
     *
     * @param bizData 业务数据
     * @return 任务信息
     */
    protected ReportApproveSyncBizApproveReqVO parseToReportApproveSyncBizApproveReqVO(Map<String, Object> bizData) {
        if (CollUtil.isNotEmpty(bizData)) {
            Object taskInfoObject = bizData.get(ApproveConstants.TASK_INFO);
            ReportApproveSyncBizApproveReqVO reportApproveSyncBizApproveReqVO = new ReportApproveSyncBizApproveReqVO();
            BeanUtil.copyProperties(taskInfoObject, reportApproveSyncBizApproveReqVO);
            return reportApproveSyncBizApproveReqVO;
        }
        return null;
    }

    /**
     * 转换要素信息
     *
     * @param bizData 业务数据
     * @return 要素信息
     */
    @SuppressWarnings("unchecked")
    protected List<ReportApproveSyncBizGridReqVO> parseToReportApproveSyncBizGridReqVO(Map<String, Object> bizData) {
        List<ReportApproveSyncBizGridReqVO> respList = Collections.emptyList();
        if (CollUtil.isNotEmpty(bizData)) {
            Object approveGridObject = bizData.get(ApproveConstants.APPROVE_GRID);

            if (approveGridObject instanceof List) {

                List<Object> approveGridObjectList = (List<Object>) approveGridObject;
                if (ObjectUtil.isNotNull(approveGridObjectList)) {
                    respList = new ArrayList<>(approveGridObjectList.size());
                    for (Object approveGridItemObject : approveGridObjectList) {
                        ReportApproveSyncBizGridReqVO approveSyncBizGridReqVO = new ReportApproveSyncBizGridReqVO();
                        BeanUtil.copyProperties(approveGridItemObject, approveSyncBizGridReqVO);

                        respList.add(approveSyncBizGridReqVO);
                    }
                }
            }
        }
        return respList;
    }

    // 20241106 add 本地审批调整成BU V2 方案 end
    /**
     * 转换
     *
     * @param approvalInfoDTO 审批单信息
     * @return 流程业务数据
     */
    protected BuApproveSyncBizApproveReqVO parseApprovalInfoDTO(ApprovalInfoDTO approvalInfoDTO) {
        Assert.notNull(approvalInfoDTO);
        BuApproveSyncBizApproveReqVO buApproveSyncBizApproveReqVO = new BuApproveSyncBizApproveReqVO();
        buApproveSyncBizApproveReqVO.setTitle(approvalInfoDTO.getProcessTitle());
        buApproveSyncBizApproveReqVO.setTaskClass(approvalInfoDTO.getApproveTaskClass());
        buApproveSyncBizApproveReqVO.setTaskStatus(TaskStatusEnum.ENABLE.value);
        buApproveSyncBizApproveReqVO.setTaskTermStart(approvalInfoDTO.getStartTime());
        buApproveSyncBizApproveReqVO.setTaskTermEnd(approvalInfoDTO.getEndTime());
        return buApproveSyncBizApproveReqVO;
    }


}
